"""Console script for zonic."""
import argparse
import socket
import sys

import colorama

from .configuration import Configuration
from .impl import Attacker
from .impl import Scanner
from .impl import Writer
from .instance import ZonicInstance
from .utility.argparsing import RangeAction


def parse_sysargv() -> argparse.Namespace:
    """
    Parses sys.argv via the argparse module.
    store_true | store_false are implicit in their values, however we are explicit here.
    :return: The argparse.Namespace instance from `parse_args()`
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-t",
        "--target",
        action="store",
        required=True,
        dest="target",
        type=socket.gethostbyname,
        metavar="localhost",
        help="Target host used for port inspection.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Increase verbosity, zonic supports a simple ON|OFF verbosity level over count(-vvv) etc.",
        dest="verbose",
        default=False,
    )
    parser.add_argument(
        "-q",
        "--quick",
        action="store_true",
        help="Scan the range of common ports, disregarding --port-range.",
        dest="quick",
    )
    parser.add_argument(
        "-i",
        "--irregular",
        action="store_true",
        help="Ports are randomly scanned (even when threading the workload) to avoid sequential execution "
        "Some systems can spot the pattern originating and activate blocking mechanism quicker.",
        dest="irregular",
    )
    parser.add_argument(
        "-tc",
        "--thread-count",
        action="store",
        type=int,
        default=1_000,
        metavar="2000",
        help="Number of threads to use for scanning, scanning is IO bound and substancial performance"
        "gains can be had (at the price of load on the target host)",
        dest="thread_count",
    )
    parser.add_argument(
        "-pr",
        "--port-range",
        action=RangeAction,
        default=range(65536),
        metavar="22-8080",
        help="Specifying a port range",
        dest="port_range",
    )
    parser.add_argument(
        "-csv",
        "--csv-output",
        action="store_true",
        default=False,
        help="Write the vulnerable ports to a csv file in the current working directory.",
        dest="csv_output",
    )
    return parser.parse_args()


def main():
    colorama.init()
    namespace = parse_sysargv()
    config = Configuration(**vars(namespace))
    scanner = Scanner(config)
    attacker = Attacker()
    zonic = ZonicInstance(
        config=config, scannable=scanner, attackable=attacker, writable=Writer()
    )
    zonic.display_configuration()
    zonic.attack()


def init() -> None:
    # unit testable
    if __name__ == "__main__":
        sys.exit(main())  # pragma: no cover


init()



import argparse
import typing

from zonic.exceptions import PortRangeValueError


class RangeAction(argparse.Action):
    def __call__(
        self,
        parser: argparse.ArgumentParser,
        namespace: argparse.Namespace,
        values: typing.Union[typing.Sequence[typing.Any], str, None],
        option_string: typing.Optional[str] = None,
    ):
        """
        This are only invoked if the respective argument we use it against is provided
        explicitly on the CLI; otherwise the default value is assumed.
        """
        assert isinstance(values, str)
        try:
            sep = "-"
            first, sep, second = values.partition(sep)
            setattr(namespace, self.dest, range(int(first), int(second)))
        except ValueError:
            raise PortRangeValueError(values) from None



from typing import Iterable


class Configuration:
    def __init__(
        self,
        target: str,
        verbose: bool,
        quick: bool,
        irregular: bool,
        thread_count: int,
        port_range: Iterable[int],
        csv_output: bool,
    ) -> None:
        self.target = target
        self.verbose = verbose
        self.quick = quick
        self.irregular = irregular
        self.thread_count = thread_count
        self.port_range = port_range
        self.csv_output = csv_output

    def __repr__(self) -> str:
        return (
            f"Configuration(target={self.target}, "
            f"verbose={self.verbose}, "
            f"quick={self.quick}, "
            f"irregular={self.irregular}, "
            f"thread_count={self.thread_count}, "
            f"port_range={self.port_range}, "
            f"csv_output={self.csv_output})"
        )



from ..abc.base_classes import Writable
from ..utility.string_utils import StringTemplate


class Writer(Writable):
    def write(self, template: StringTemplate, flush: bool = True) -> None:
        print(str(template), flush=True)



import typing

from zonic.abc.base_classes import Scanable
from zonic.configuration import Configuration


class Scanner(Scanable):
    def __init__(self, config: Configuration):
        self.config = config

    def scan(self, port: int) -> typing.Tuple[int, bool]:
        ...



import typing


class StringTemplate:
    """
    Encapsulating colourised string messages that can be written to
    standard out, when inspected the instance the colours are
    substituted into the string, for use in colourised output.

    :
    """

    def __init__(
        self,
        message: str,
        colour_options: typing.Optional[typing.Tuple[str, ...]] = None,
    ) -> None:
        self.message = message
        self.colour_options = colour_options

    def __repr__(self) -> str:
        return (
            self.message
            if not self.colour_options
            else self.message.format(*self.colour_options)
        )



import subprocess

import pytest
from assertpy import assert_that

from zonic.exceptions import PortRangeValueError


@pytest.mark.parametrize(
    "port_range, expected",
    [("10-100s", 1), ("", 1), ("fail", 1), ("20 8080", 1), ("20-8080", 0)],
)
def test_exit_codes_for_port_range(port_range, expected) -> None:
    exit_code = 0
    try:
        subprocess.check_output(["zonic", "-t", "localhost", "-pr", port_range])
    except subprocess.CalledProcessError as err:
        exit_code = err.returncode
    assert_that(exit_code).is_equal_to(expected)


def test_port_range_exc_message() -> None:
    expected = (
        "Invalid value specified for --port-range: `fail=true`,"
        " correct usage is: `--port-range | -pr 20-8080`"
    )
    with pytest.raises(PortRangeValueError) as err:
        raise PortRangeValueError(value="fail=true")
    assert_that(err.value.message).is_equal_to(expected)  # type: ignore



import colorama

from .abc.base_classes import Attackable
from .abc.base_classes import Scanable
from .abc.base_classes import Writable
from .configuration import Configuration
from .utility.string_utils import StringTemplate


class ZonicInstance:
    def __init__(
        self,
        config: Configuration,
        scannable: Scanable,
        attackable: Attackable,
        writable: Writable,
    ) -> None:
        self.config = config
        self._scannable = scannable
        self._attackable = attackable
        self._writable = writable
        self._welcome()

    def _welcome(self) -> None:
        """
        Write the welcome colourised ASCII art to standard art
        """
        template = StringTemplate(
            message=r"""
{0} ================================================= {1}{2}
 ______     ______     __   __     __     ______
/\___  \   /\  __ \   /\ "-.\ \   /\ \   /\  ___\
\/_/  /__  \ \ \/\ \  \ \ \-.  \  \ \ \  \ \ \____
  /\_____\  \ \_____\  \ \_\\"\_\  \ \_\  \ \_____\
  \/_____/   \/_____/   \/_/ \/_/   \/_/   \/_____/
{3}""",
            colour_options=(
                colorama.Fore.BLUE,
                colorama.Fore.RESET,
                colorama.Fore.LIGHTYELLOW_EX,
                colorama.Fore.RESET,
            ),
        )
        self.write(template)

    def display_configuration(self) -> None:
        self.write(
            StringTemplate(
                "{0}={1}" * 50, colour_options=(colorama.Fore.BLUE, colorama.Fore.RESET)
            )
        )
        self.write(
            StringTemplate(
                "Zonic: Powerful python port scanner, https://github.com/symonk/zonic"
            )
        )
        for opt, val in self.config.__dict__.items():
            self.write(
                StringTemplate(
                    "    Option: {0}{1}{2} => {3}",
                    colour_options=(colorama.Fore.GREEN, opt, colorama.Fore.RESET, val),
                )
            )
        self.write(
            StringTemplate(
                "{0}={1}" * 50, colour_options=(colorama.Fore.BLUE, colorama.Fore.RESET)
            )
        )

    def execute(self) -> int:
        ...

    def scan(self) -> None:
        ...

    def attack(self) -> None:
        ...

    def write(self, template: StringTemplate, flush: bool = True) -> None:
        self._writable.write(template=template, flush=flush)



import pytest
from assertpy import assert_that

from zonic.utility.string_utils import StringTemplate


@pytest.mark.parametrize(
    "template, expected",
    [
        (StringTemplate(message="a{0}", colour_options=(" b",)), "a b"),
        (StringTemplate(message="foo", colour_options=None), "foo"),
    ],
)
def test_string_template(template, expected) -> None:
    assert_that(str(template)).is_equal_to(expected)



class PortRangeValueError(ValueError):
    def __init__(self, value: str) -> None:
        super().__init__(self)
        self.message = (
            f"Invalid value specified for --port-range: `{value}`, "
            f"correct usage is: `--port-range | -pr 20-8080`"
        )



import abc
import typing

from zonic.utility.string_utils import StringTemplate


class Attackable(abc.ABC):
    @abc.abstractmethod
    def attack(
        self, port_range: typing.Iterable[int]
    ) -> typing.Optional[typing.Iterable[int]]:
        """
        Begin attacking ports in the port range
        """
        ...


class Scanable(abc.ABC):
    @abc.abstractmethod
    def scan(self, port: int) -> typing.Tuple[int, bool]:
        """
        Perform a port scan against a particular port.
        :param port: The port (integer) to perform the scan against
        :return: A tuple of (port_number, boolean to indicate if port was considered `open`
        """
        ...


class Writable(abc.ABC):
    @abc.abstractmethod
    def write(self, template: StringTemplate, flush: bool = True) -> None:
        """
        Write messages to standard out
        :param template: StringTemplate instance to be written to stdout
        :param flush: A boolean flag to indicate if a flush should be performed after
        """
        ...



import typing

from zonic.abc.base_classes import Attackable


class Attacker(Attackable):
    def attack(
        self, port_range: typing.Iterable[int]
    ) -> typing.Optional[typing.Iterable[int]]:
        ...
